#
# Copyright (c) 2021-2025 ahriman team.
#
# This file is part of ahriman
# (see https://github.com/arcan1s/ahriman).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
from pathlib import Path

from ahriman.core.exceptions import BuildError
from ahriman.core.log import LazyLogging
from ahriman.core.utils import check_output
from ahriman.models.repository_paths import RepositoryPaths


class Repo(LazyLogging):
    """
    repo-add and repo-remove wrapper

    Attributes:
        name(str): repository name
        sign_args(list[str]): additional args which have to be used to sign repository archive
        uid(int): uid of the repository owner user
    """

    def __init__(self, name: str, paths: RepositoryPaths, sign_args: list[str]) -> None:
        """
        Args:
            name(str): repository name
            paths(RepositoryPaths): repository paths instance
            sign_args(list[str]): additional args which have to be used to sign repository archive
        """
        self.name = name
        self.uid, _ = paths.root_owner
        self.sign_args = sign_args

    def add(self, path: Path, root: Path, *, remove_previous: bool) -> None:
        """
        add new package to repository

        Args:
            path(Path): path to archive to add
            root(Path): path to repository root
            remove_previous(bool): whether to remove old package or not
        """
        command = ["repo-add", *self.sign_args]
        if remove_previous:
            command.append("--remove")
        command.extend([str(self.repo_path(root)), str(path)])

        check_output(
            *command,
            exception=BuildError.from_process(path.name),
            cwd=root,
            logger=self.logger,
            user=self.uid,
        )

    def init(self, root: Path) -> None:
        """
        create empty repository database. It just calls add with empty arguments

        Args:
            root(Path): path to repository root
        """
        check_output("repo-add", *self.sign_args, str(self.repo_path(root)),
                     cwd=root, logger=self.logger, user=self.uid)

    def remove(self, package: str, filename: Path, root: Path) -> None:
        """
        remove package from repository

        Args:
            package(str): package name to remove
            filename(Path): package filename to remove
            root(Path): path to repository root
        """
        # remove package and signature (if any) from filesystem
        for full_path in root.glob(f"**/{filename}*"):
            full_path.unlink()

        # remove package from registry
        check_output(
            "repo-remove", *self.sign_args, str(self.repo_path(root)), package,
            exception=BuildError.from_process(package),
            cwd=self.paths.repository,
            logger=self.logger,
            user=self.uid,
        )

    def repo_path(self, root: Path) -> Path:
        """
        get full path to the repository database

        Args:
            root(Path): repository root

        Returns:
            Path: path to repository database
        """
        return root / f"{self.name}.db.tar.gz"
